# OB - write your own commands.
#
# handler (hdl)

"tasks."

# imports

import queue, threading, time, _thread

from olib import Object, get, get_name, update
from ob.prs import elapsed
from ob.utl import get_exception

starttime = time.time()

# classes

class Task(threading.Thread):

    "A task is a function run in a thread."

    def __init__(self, func, *args, name="noname", daemon=True):
        super().__init__(None, self.run, name, (), {}, daemon=daemon)
        self._name = name
        self._result = None
        self._queue = queue.Queue()
        self._queue.put((func, args))
        self.sleep = 0
        self.state = Object()

    def __iter__(self):
        return self

    def __next__(self):
        for k in dir(self):
            yield k

    def run(self):
        "run the tasks function."
        func, args = self._queue.get()
        self.setName(self._name)
        try:
            self._result = func(*args)
        except EOFError:
            _thread.interrupt_main()
        except Exception as _ex:
            print(get_exception())

    def join(self, timeout=None):
        "join the task."
        super().join(timeout)
        return self._result

# utilities

def launch(func, *args, **kwargs):
    "launch a task to run function in."
    name = kwargs.get("name", get_name(func))
    t = Task(func, *args, name=name, daemon=True)
    t.start()
    return t

# commands

def tsk(event):
    "show running tasks."
    psformat = "%-8s %-50s"
    result = []
    for thr in sorted(threading.enumerate(), key=lambda x: x.getName()):
        if str(thr).startswith("<_"):
            continue
        d = vars(thr)
        o = Object()
        update(o, d)
        state = get(o, "state", Object())
        up = get(o, "sleep", time.time()) - int(time.time()-get(state, "latest", starttime))
        result.append((up, thr.getName(), o))
    nr = -1
    for up, thrname, o in sorted(result, key=lambda x: x[0]):
        nr += 1
        res = "%s %s" % (nr, psformat % (elapsed(up), thrname[:60]))
        if res:
            event.reply(res.rstrip())
