# OB - write your own commands.
#
# utilities (utl)

"utilities module."

# imports

import importlib, inspect, os, sys, traceback

from olib import ENOCLASS, Object, Ol

# utilities

def direct(name):
    "do a direct import."
    return importlib.import_module(name)

def find_cmds(mod):
    "scan module for commands."
    cmds = {}
    for key, o in inspect.getmembers(mod, inspect.isfunction):
        if "event" in o.__code__.co_varnames:
            if o.__code__.co_argcount == 1:
                cmds[key] = o
    return cmds

def find_modules(pkgs, skip=None):
    "scan package for modules."
    modules = []
    for pkg in pkgs.split(","):
        if skip is not None and skip not in pkg:
            continue
        try:
            p = direct(pkg)
        except ModuleNotFoundError:
            continue
        for _key, m in inspect.getmembers(p, inspect.ismodule):
            if m not in modules:
                modules.append(m)
    return modules

def find_shorts(mn):
    "find lowernamed class names."
    shorts = Ol()
    for mod in find_modules(mn):
        for _key, o in inspect.getmembers(mod, inspect.isclass):
            if issubclass(o, Object):
                t = "%s.%s" % (o.__module__, o.__name__)
                shorts.append(o.__name__.lower(), t)
    return shorts

def list_files(wd):
    "wd is directory to list files of."
    path = os.path.join(wd, "store")
    if not os.path.exists(path):
        return ""
    return "|".join(os.listdir(path))

def get_cls(name):
    "name is classname to create class from."
    try:
        modname, clsname = name.rsplit(".", 1)
    except:
        raise ENOCLASS(name)
    if modname in sys.modules:
        mod = sys.modules[modname]
    else:
        mod = importlib.import_module(modname)
    return getattr(mod, clsname)

def get_exception(txt="", sep="|"):
    "print exception trace in one line."
    exctype, excvalue, tb = sys.exc_info()
    trace = traceback.extract_tb(tb)
    result = []
    for elem in trace:
        if elem[0].endswith(".py"):
            plugfile = elem[0][:-3].split(os.sep)
        else:
            plugfile = elem[0].split(os.sep)
        mod = []
        print(plugfile)
        for element in plugfile[::-1]:
            mod.append(element)
        ownname = ".".join(mod[::-1])
        result.append("%s:%s" % (ownname, elem[1]))
    res = "%s %s: %s %s" % (sep.join(result), exctype, excvalue, str(txt))
    del trace
    return res

def root():
    "see if program is run as root."
    if os.geteuid() != 0:
        return False
    return True

def spl(txt):
    "split comma seperated text."
    return iter([x for x in txt.split(",") if x])
